package cz.vsb.jtestserver.service;

import org.apache.log4j.Logger;
import org.apache.struts.upload.FormFile;

import cz.vsb.jtestserver.util.CompileFile;
import cz.vsb.jtestserver.util.Configuration;
import cz.vsb.jtestserver.util.Database;
import cz.vsb.jtestserver.util.FileUpload;
import cz.vsb.jtestserver.util.TestUserFile;
import cz.vsb.jtestserver.view.CviceniView;
import cz.vsb.jtestserver.view.PredmetView;
import cz.vsb.jtestserver.view.UlohaView;
import cz.vsb.jtestserver.view.UserView;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

public class FileUploadManager {

	static Logger logger = Logger.getLogger(FileUploadManager.class);

	private UserView userView;

	private PredmetView predmetView;
	
	private String saveFileTPath;

	private String saveFilePath;
	
	private String saveTestPath;
	
	private String junitPath;
	
	public FileUploadManager(UserView uv, PredmetView pv) {
		userView = uv;
		predmetView = pv;
		saveFileTPath = Configuration.getProperty("saveFileTPath"); 
		saveFilePath = Configuration.getProperty("saveFilePath");
		saveTestPath = Configuration.getProperty("saveTestPath");
		junitPath = Configuration.getProperty("junitPath");
	}

	public void fileUpload(FormFile formFile, int idUlohy)
			throws InvalidFileNameException, FileNotCompiledException,
			FileNotLoadedException {
		logger.debug("Jmeno souboru z formulare " + formFile.getFileName());
		logger.debug("Jmeno souboru z konfigurace " + saveFileTPath);
		logger.debug("Predmet " + predmetView.getZkratka());
		logger.debug("Nazev ulohy: " + idUlohy);

		// jmeno souboru pro porovnani se jmenem tridy
		String formFileName = formFile.getFileName();
		String className = formFileName.substring(0, formFileName
				.lastIndexOf("."));

		UlohaManager ulohaM = new UlohaManager(userView);
		UlohaView ulohaView = ulohaM.findUloha(idUlohy);

		if (className.equals(ulohaView.getJmenoTridy())) {
			StringBuilder savePathB = new StringBuilder();
			if (!userView.getStatus().equals("student")) {
				savePathB.append(saveFileTPath);
				savePathB.append(File.separator);
				savePathB.append(predmetView.getZkratka());
				savePathB.append(File.separator);
				savePathB.append(ulohaView.getNazevUlohy());
			} else {
				savePathB.append(saveFilePath);
				savePathB.append(File.separator);
				savePathB.append(predmetView.getZkratka());
				savePathB.append(File.separator);
				savePathB.append(ulohaView.getNazevUlohy());
			}
			String savePath = savePathB.toString();
			logger.debug("Cesta pro ulozeni: " + savePath);
			FileUpload fUpload = new FileUpload();
			boolean success;
			try {
				success = fUpload.doUpload(formFile.getInputStream(), formFile
						.getFileName(), savePath);
			} catch (FileNotFoundException e) {
				logger.error(e.getMessage());
				success = false;
			} catch (IOException e) {
				logger.error(e.getMessage());
				success = false;
			}

			if (success) {
				CompileFile compileFile = new CompileFile();
				boolean result = compileFile.doCompile(savePath, savePath,
						savePath + File.separator + formFile.getFileName());
				if (result) {
					ulohaView.setKompilace("A");
					ulohaM.updateUlohaKompilace(idUlohy, ulohaView
							.getKompilace());
				} else {
					String message = "Soubor nelze zkompilovat";
					logger.warn(message);
					throw new FileNotCompiledException(message);
				}
			} else {
				String message = "Soubor nelze nahrat";
				logger.warn(message);
				throw new FileNotLoadedException(message);
			}
		} else {
			logger.warn("Jmeno souboru nesouhlasi se jmenem tridy");
			throw new InvalidFileNameException("Neplatne jmeno souboru");
		}
	}

	public void fileStudentUpload(FormFile formFile, CviceniView cviceniView, 
			UlohaView ulohaView) throws InvalidFileNameException, 
			FileNotCompiledException, FileNotLoadedException {
		logger.debug("Jmeno souboru z formulare " + formFile.getFileName());
		logger.debug("Jmeno souboru z konfigurace " + saveFileTPath);
		logger.debug("Predmet " + predmetView.getZkratka());

		// jmeno souboru pro porovnani s Jmenem Tridy
		String formFileName = formFile.getFileName();
		String className = formFileName.substring(0, formFileName
				.lastIndexOf("."));

		StringBuilder sb = new StringBuilder();
		sb.append(saveFilePath);
		sb.append(File.separator);
		sb.append(predmetView.getZkratka());
		sb.append(File.separator);
		sb.append(ulohaView.getNazevUlohy());
		sb.append(File.separator);
		sb.append(userView.getLogin());
		String savePath = sb.toString();
		
		sb = new StringBuilder();
		sb.append(saveTestPath);
		sb.append(File.separator);
		sb.append(predmetView.getZkratka());
		sb.append(File.separator);
		sb.append(ulohaView.getNazevUlohy());
		String testPath = sb.toString();

		// Vytvoreni adresase pro pridatni testovanych souboru od studenta
		// TestServer/Data/Student/<nazev ulohy>/<login - student>
		File fileAdr = new File(savePath);
		if (fileAdr.exists() == false) {
			fileAdr.mkdir();
		}

		FileUpload fUpload = new FileUpload();
		boolean success;

		// zapis prazdnych dat do tabulky a zjisteni testovanych souboru
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		String datum = sdf.format(new Date());
		Connection con = null;
		try {
			con = Database.getConnection();
			String query = "INSERT INTO vysledky VALUES(null, ?, ?, ?, ?, ?, 'N', 0, 0, 0)";
			logger.debug(query);
			PreparedStatement sql = con.prepareStatement(query);
			sql.setString(1, userView.getLogin());
			sql.setInt(2, cviceniView.getIdRozvrhu());
			sql.setInt(3, cviceniView.getIdCviceni());
			sql.setInt(4, ulohaView.getIdUlohy());
			sql.setString(5, datum);
			sql.executeUpdate();
                        sql.close();
		} catch (SQLException e) {
			logger.error(e, e);
		}

		int idVysledku = 0;
		try {
			con = Database.getConnection();
			String query = "SELECT id_vysledku FROM Vysledky WHERE login = ? AND odevzdano = ?";
			logger.debug(query);
			PreparedStatement sql = con.prepareStatement(query);
			sql.setString(1, userView.getLogin());
			sql.setString(2, datum);
			ResultSet result = sql.executeQuery();
			if (result.next()) {
				idVysledku = result.getInt(1);
			}
                        result.close();
                        sql.close();                        
		} catch (SQLException e) {
			logger.error(e, e);
		}

		ArrayList<String> testy = new ArrayList<String>();
		try {
			con = Database.getConnection();
			String query = "SELECT nazev_testu FROM testy WHERE id_ulohy = "
				+ ulohaView.getIdUlohy();
			logger.debug(query);
			Statement sql = con.createStatement();
			ResultSet result = sql.executeQuery(query);
			while (result.next()) {
				testy.add(result.getString(1));
			}
                        result.close();
                        sql.close();
                        con.close();
		} catch (SQLException e) {
			logger.error(e, e);
		}
		logger.debug("Pocet testu: " + testy.size());

		// zacatek testu

		if (className.equals(ulohaView.getJmenoTridy())) {
			try {
				success = fUpload.doUpload(formFile.getInputStream(), formFile
						.getFileName(), savePath);
			} catch (FileNotFoundException fnfe) {
				success = false;
			} catch (IOException ioe) {
				success = false;
			}
			// nahrani skoncilo
			if (success) {
                          try {
                            CompileFile compileFile = new CompileFile();
                            boolean result = compileFile.doCompile(savePath, savePath,
                                          savePath + File.separator + formFile.getFileName());
                            if (result) {
                                  // kompilace uspesna mohou probehnou testy
                                  TestUserFile tuf = new TestUserFile();
                                  logger.debug("Uspesny preklad u studenta " + userView.getLogin());
                                  success = tuf.doTest(savePath + File.separator, testPath
                                                  + File.separator, junitPath, testy,
                                                  userView.getLogin(), ulohaView.getNazevUlohy(), userView
                                                                  .getStatus(), idVysledku);
                            } 
                          } catch (FileNotCompiledException ex) {
                            logger.error(ex.getMessage());
                            throw new FileNotCompiledException("Soubor nelze zkompilovat",ex);
                          }
			} else {
				try {
					con = Database.getConnection();
					String query = "UPDATE vysledky SET pocet_eroru = 1 WHERE id_vysledku = ?";
					logger.debug(query);
					PreparedStatement sql = con.prepareStatement(query);
					sql.setInt(1, idVysledku);
					sql.executeUpdate();
                                        sql.close();
				} catch (SQLException e) {
					logger.error(e, e);
				}
				String message = "Soubor nelze nahrat";
				logger.error(message);
				throw new FileNotLoadedException(message);
			}
		} else {
			try {
				con = Database.getConnection();
				String query = "UPDATE vysledky SET pocet_eroru = 1 WHERE id_vysledku = ?";
				logger.debug(query);
				PreparedStatement sql = con.prepareStatement(query);
				sql.setInt(1, idVysledku);
				sql.executeUpdate();
                                sql.close();
			} catch (SQLException e) {
				logger.error(e, e);
			}
			String message = "Neplatne jmeno souboru";
			logger.error(message);
			throw new InvalidFileNameException(message);
		}
                try {
                  con.close();
                } catch (SQLException ex) {
                  logger.error(ex);
                }
	}
}
