package cz.vsb.jtestserver.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.List;

import org.apache.log4j.Logger;

import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

/**
 * Trida, ktera provadi testy
 * 
 * @author Martin Lason
 * @author David Sousedik
 */
public class Test extends Thread {

	Logger logger = Logger.getLogger(Test.class);

	public Test(ThreadGroup threadGroup, String threadName, List<String> tests,
			String filePath, String testPath, String junitPath, String login,
			String taskName) {
		super(threadGroup, threadName);
		this.login = login;
		this.taskName = taskName;
		this.tests = tests;
		this.saveFilePath = filePath;
		this.saveTestPath = testPath;
		this.junitPath = junitPath;
		this.webserverPath = Configuration.getProperty("webserverPath");
		stopped = false;
	}

	/**
	 * Nastavuje mnozinu testu
	 */
	public void setEnumTests(Enumeration enumeration) {
		this.testsEnum = enumeration;
	}

	/**
	 * Ziskava mnozinu testu
	 */
	public Enumeration getEnumTests() {
		return testsEnum;
	}

	/**
	 * Nastavuje objekt vysledku testu
	 */
	public void setTestResult(TestResult testResult) {
		this.testResult = testResult;
	}

	/**
	 * Ziskava objekt vysledku testu
	 */
	public TestResult getTestResult() {
		return testResult;
	}

	/**
	 * Nastavuje pocet failu
	 */
	public void setFailureCount(int failureCount) {
		this.failureCount = failureCount;
	}

	/**
	 * Ziskava pocet failu
	 */
	public int getFailureCount() {
		return failureCount;
	}

	/**
	 * Nastavuje pocet erroru
	 */
	public void setErrorCount(int errorCount) {
		this.errorCount = errorCount;
	}

	/**
	 * Ziskava pocet erroru
	 */
	public int getErrorCount() {
		return errorCount;
	}

	/**
	 * Informace, ze je vlakno stopnuto
	 */
	public void stopped() {
		stopped = true;
	}

	/**
	 * Je vlakno stopnuto?
	 */
	public boolean isStopped() {
		return stopped;
	}

	/**
	 * Metoda, ktera provadi testy
	 */
	public void run() {
		URLClassLoader urlclassloader = null;
		try {
			URL url1 = new URL("file:///" + saveTestPath);
			URL url = new URL("file:///" + saveFilePath);
			URL url2 = new URL("file:///" + junitPath);
			ClassLoader classloader = getClass().getClassLoader();
			urlclassloader = new URLClassLoader(new URL[] { url1, url, url2 },
					classloader);
		} catch (MalformedURLException e) {
			logger.error(e, e);
			return;
		}

		testResult = new TestResult();
		testSuite = new TestSuite();

		logger.debug("Jmeno TestSuite: " + testSuite.getName());

		Calendar calendar = Calendar.getInstance();
		// datum
		StringBuilder sb = new StringBuilder(32);
		int day = calendar.get(Calendar.DAY_OF_MONTH);
		if (day < 10)
			sb.append('0');
		sb.append(day);
		sb.append('.');
		int mon = calendar.get(Calendar.MONTH);
		if (mon < 10)
			sb.append('0');
		sb.append(mon);
		sb.append('.');
		sb.append(calendar.get(Calendar.YEAR));
		String date = sb.toString();

		// cas
		sb = new StringBuilder(32);
		int hour = calendar.get(Calendar.HOUR_OF_DAY);
		if (hour < 10)
			sb.append('0');
		sb.append(hour);
		sb.append(':');
		int min = calendar.get(Calendar.MINUTE);
		if (min < 10)
			sb.append('0');
		sb.append(min);
		sb.append(':');
		int sec = calendar.get(Calendar.SECOND);
		if (sec < 10)
			sb.append('0');
		sb.append(sec);
		String time = sb.toString();

		XMLDocumentCreator xmlDocument = new XMLDocumentCreator();
		xmlDocument.startTestSuites(taskName, login, time, date);
		logger.debug("Vytvarim XML dokument " + taskName + " pro " + login);

		sb = new StringBuilder(255);
		sb.append(saveFilePath);
		sb.append(taskName);
		sb.append('-');
		sb.append(hour);
		sb.append('-');
		sb.append(min);
		sb.append('-');
		sb.append(sec);
		sb.append(".xml");

		FileOutputStream fileoutputstream = null;
		try {
			fileoutputstream = new FileOutputStream(new File(sb.toString()));
		} catch (FileNotFoundException e) {
			logger.error(e, e);
			return;
		}

		int totalTests = 0;
		int totalFails = 0;
		int totalErrors = 0;
		xmlDocument.setOutput(fileoutputstream);
		for (String testName : tests) {
			if (stopped) {
				logger.warn("Vlakno nasilne ukonceno");
				return;
			}
			Class testClass;
			try {
				testClass = urlclassloader.loadClass(testName);
			} catch (ClassNotFoundException e) {
				logger.error(e, e);
				return;
			}
			testSuite = new TestSuite((Class<? extends TestCase>) testClass);
			xmlDocument.startTestSuite(testSuite);
			for (Enumeration testEnum = testSuite.tests(); testEnum
					.hasMoreElements();) {
				logger.debug(testEnum.nextElement());
			}
			int numTestCases = testSuite.countTestCases();
			for (int i = 0; i < numTestCases; i++) {
				TestCase testcase = (TestCase) testSuite.testAt(i);
				testResult = new TestResult();
				xmlDocument.startTest(testcase);
				testcase.run(testResult);
				xmlDocument.endTest(testcase, testResult);
				int testFailCount = testResult.failureCount();
				int testErrorCount = testResult.errorCount();
				totalFails += testFailCount;
				totalErrors += testErrorCount;
				if (testResult.wasSuccessful()) {
					logger.debug("Testy probehly v poradku");
					continue;
				} else {
					logger.debug("V testech se vyskytly chyby");
					for (Enumeration e = testResult.failures(); e
							.hasMoreElements();) {
						logger.debug(e.nextElement());
					}
					for (Enumeration e = testResult.errors(); e
							.hasMoreElements();) {
						logger.debug(e.nextElement());
					}
				}
			}

			totalTests += numTestCases;
			xmlDocument.endTestSuite(testSuite);
		}

		logger.debug("Pocet testu celkem: " + totalTests);
		logger.debug("Pocet failu celkem : " + totalFails);
		logger.debug("Pocet erroru celkem: " + totalErrors);
		setFailureCount(totalFails);
		setErrorCount(totalErrors);
		xmlDocument.endTestSuites(totalTests, totalFails, totalErrors);

		logger.debug("Pocet testu: " + testSuite.testCount());

		testsEnum = testSuite.tests();

		setEnumTests(testsEnum);
		testSuite.run(testResult);
		logger.debug("Pocet testu: " + testResult.runCount());
		if (testResult.wasSuccessful()) {
			logger.debug("Testy byly uspesne");
		} else {
			logger.debug("Testy nebyly uspesne");
		}
		try {
			copyResults(sb.toString());
		} catch (IOException e) {
			logger.error(e, e);
		}
		setTestResult(testResult);
	}

	private void copyResults(String fileName) throws IOException {
		logger.debug("Kopirovani vysledku z " + fileName);
		File srcFile = new File(fileName);
		if (!srcFile.isFile()) {
			return;
		}
		File destFile = new File(webserverPath + File.separator + login + ".xml");
		if (destFile.exists())
			destFile.delete();
		RandomAccessFile zdroj = new RandomAccessFile(srcFile, "r");
		RandomAccessFile cil = new RandomAccessFile(destFile, "rw");
		for (int n = 0; n < srcFile.length(); n++)
			cil.writeByte(zdroj.readByte());
		zdroj.close();
		cil.close();
		logger.debug("Data prekopirovana do " + destFile);
	}

	private boolean stopped;

	private String login;

	private List<String> tests;

	private TestResult testResult;

	private TestSuite testSuite;

	private Enumeration testsEnum;

	private String saveFilePath;

	private String saveTestPath;

	private String junitPath;
	
	private String webserverPath;

	private String taskName;

	private int failureCount;

	private int errorCount;
}
